-- Use Write-Ahead-Logging --
-- This speeds up queries by about 2x --
-- PRAGMA journal_mode=WAL;
-- PRAGMA synchronous = OFF; -- Disabled by default, can corrupt db if the computer suffers a catastrophic crash (or power failure) 

-- This drops the existing files table, and re-creates it. --
DROP TABLE IF EXISTS 'files';
CREATE TABLE IF NOT EXISTS 'files' (
            file TEXT NOT NULL,
            mimetype TEXT NOT NULL,
            expiry INTEGER NOT NULL,
            expiry_override INTEGER,
            views INTEGER DEFAULT '0',
            is_deleted INTEGER DEFAULT '0',
            owner TEXT NOT NULL,
            uploaded INTEGER NOT NULL,
            last_access INTEGER NOT NULL,
            filesize INTEGER NOT NULL,
            IP TEXT NOT NULL,
            domain TEXT NOT NULL);

-- This drops the existing qrscan table, and re-creates it. --
DROP TABLE IF EXISTS 'qrscan';
CREATE TABLE 'qrscan' (
            scanid INTEGER PRIMARY KEY,
            time INTEGER NOT NULL,
            IP TEXT NOT NULL,
            useragent TEXT NOT NULL,
            version INTEGER NOT NULL
        );


-- API Key Table - This is the first time I've used Foreign keys since uni. wow. --
CREATE TABLE IF NOT EXISTS 'keys' (
            uuid TEXT NOT NULL PRIMARY KEY,
            key_level INTEGER NOT NULL,
            parent_key TEXT,
            revoked TEXT,
            creation_time INTEGER NOT NULL,
            comment TEXT,
            FOREIGN KEY (parent_key) REFERENCES keys(uuid)
        );

-- View showing keys and json list of their children --
CREATE VIEW IF NOT EXISTS keys_with_children AS
SELECT
    parent.uuid AS uuid,
    parent.key_level AS key_level,
    parent.parent_key AS parent_key,
    parent.revoked AS revoked,
    parent.creation_time AS creation_time,
    parent.comment AS comment,
    json_group_array(child.uuid) AS child_uuids
FROM keys parent
LEFT JOIN keys child ON parent.uuid = child.parent_key
GROUP BY parent.uuid
ORDER BY parent.creation_time;

-- Dummy file to stop sqlx compile error --
-- This is some seriously wacked up formatting. --
-- 2024-05-22: I don't think this is a temporary fix anymore. --
INSERT INTO files (
                         file,     mimetype,    expiry,  owner,           last_access, uploaded, filesize, ip,          domain)
                VALUES ( 'dummy', 'text/plain', '1',     'dummyownerkey', '0',         '0',      '0',      '127.0.0.1', 'localhost');