use salvo::{handler, hyper::header::HOST, prelude::StatusCode, Request, Response};
use std::path::PathBuf;
use tracing::{debug, info};

use crate::handlers::{guess_ip, render_template, Generic, TemplateEnum};

/// This file handles (heh) the main index page for the application.
#[handler]
pub async fn index(req: &mut Request, res: &mut Response) {
    // Get the headers (for Host header stuff thats needed later)
    let headers = req.headers();
    let remote_addr = &req.remote_addr().clone();
    let domain = headers[HOST].to_str().unwrap_or("localhost:8282");
    info!(
        "[{:?} - {:?}] New Request: /",
        headers[HOST].to_str().unwrap(),
        guess_ip(headers, remote_addr)
    );

    // Renders the template based on `/templates/$host/upload.html`
    let template_host_path = PathBuf::from("./templates/").join(domain);
    debug!("template_with_host: {:?}", template_host_path);

    let template = TemplateEnum::Generic(Generic {
        domain: String::from(headers[HOST].to_str().unwrap()),
        ..Default::default()
    });
    render_template(res, headers, "upload.html", template, StatusCode::OK).await
}
