use ramhorns::Ramhorns;
use salvo::{
    handler,
    hyper::header::{HOST, USER_AGENT},
    prelude::StatusCode,
    writing::Text,
    Request, Response,
};
use std::path::PathBuf;
use tracing::{error, info, warn};

use super::guess_ip;
use crate::{
    db,
    handlers::{Generic, TemplateEnum},
    SQLITE,
};

/// This handles the requests for all static files
#[handler]
pub async fn serve_static(req: &mut Request, res: &mut Response) {
    let headers = &req.headers().clone();
    let remote_addr = &req.remote_addr().clone();
    let host = headers[HOST].to_str().unwrap_or("localhost:8282");
    let template_host_path = PathBuf::from("./templates/").join(host);
    let template_host_path_default = PathBuf::from("./templates/localhost:8282");
    let ip = guess_ip(headers, remote_addr);
    let useragent = headers[USER_AGENT].to_str().unwrap();

    // Create the shared template from the template folder.
    let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
        Ramhorns::from_folder(template_host_path_default).expect("Couldn't find default template"),
    );

    // Render each static endpoint.
    match req.uri().path() {
        "/services" => {
            info!(
                "[{:?} - {:?}] New Request: /services",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr)
            );

            // Create the template from the generic.
            let template = TemplateEnum::Generic(Generic {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            });

            // This is annoying, but since the template can be either of these, we need to check it.
            let rendered = match template {
                TemplateEnum::Generic(ref data) => tpls.get("services.html").unwrap().render(data),
                // And you may ask yourself: "Well, how did I get here?!"
                _ => {
                    panic!("Cosmic ray detected: Well, how did I get here?!")
                }
            };

            res.status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/about" => {
            info!(
                "[{:?} - {:?}] New Request: /about",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr)
            );

            // Create the template from the generic.
            let template = TemplateEnum::Generic(Generic {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            });

            // This is annoying, but since the template can be either of these, we need to check it.
            let rendered = match template {
                TemplateEnum::Generic(ref data) => tpls.get("about.html").unwrap().render(data),
                // And you may ask yourself: "Well, how did I get here?!"
                _ => {
                    panic!("Cosmic ray detected: Well, how did I get here?!")
                }
            };

            res.status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/faq" => {
            info!(
                "[{:?} - {:?}] New Request: /faq",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr)
            );

            // Create the template from the generic.
            let template = TemplateEnum::Generic(Generic {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            });

            // This is annoying, but since the template can be either of these, we need to check it.
            let rendered = match template {
                TemplateEnum::Generic(ref data) => tpls.get("faq.html").unwrap().render(data),
                // And you may ask yourself: "Well, how did I get here?!"
                _ => {
                    panic!("Cosmic ray detected: Well, how did I get here?!")
                }
            };

            res.status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/dmca" => {
            info!(
                "[{:?} - {:?}] New Request: /dmca",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr)
            );

            // Create the template from the generic.
            let template = TemplateEnum::Generic(Generic {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            });

            // This is annoying, but since the template can be either of these, we need to check it.
            let rendered = match template {
                TemplateEnum::Generic(ref data) => tpls.get("dmca.html").unwrap().render(data),
                // And you may ask yourself: "Well, how did I get here?!"
                _ => {
                    panic!("Cosmic ray detected: Well, how did I get here?!")
                }
            };

            res.status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/welcome" => {
            info!(
                "[{:?} - {:?}] New Request: /welcome",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr)
            );

            // Create the template from the generic.
            let template = TemplateEnum::Generic(Generic {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            });

            // This is annoying, but since the template can be either of these, we need to check it.
            let rendered = match template {
                TemplateEnum::Generic(ref data) => tpls.get("welcome.html").unwrap().render(data),
                // And you may ask yourself: "Well, how did I get here?!"
                _ => {
                    panic!("Cosmic ray detected: Well, how did I get here?!")
                }
            };

            res.status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/czb" => {
            info!(
                "[{:?} - {:?}] New Request: /czb",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr)
            );

            // Create the template from the generic.
            let template = TemplateEnum::Generic(Generic {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            });

            // This is annoying, but since the template can be either of these, we need to check it.
            let rendered = match template {
                TemplateEnum::Generic(ref data) => tpls.get("czb.html").unwrap().render(data),
                // And you may ask yourself: "Well, how did I get here?!"
                _ => {
                    panic!("Cosmic ray detected: Well, how did I get here?!")
                }
            };

            res.status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/qr" => {
            info!(
                "[{:?} - {:?}] New Request: /qr",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr)
            );
            // Setup the Sqlite pool stuff for later
            let sqlconn = SQLITE.get().unwrap();
            // Get the headers (for Host header stuff thats needed later)
            let headers = req.headers().clone();

            if db::add_qrscan(sqlconn, ip, useragent.to_string(), "1".to_string())
                .await
                .is_err()
            {
                error!(
                    "[{:?} - {:?}] Error when adding QR Code scan to the database",
                    headers[HOST].to_str().unwrap(),
                    guess_ip(&headers, remote_addr)
                );
            }

            // Create the template from the generic.
            let template = TemplateEnum::Generic(Generic {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            });

            // This is annoying, but since the template can be either of these, we need to check it.
            let rendered = match template {
                TemplateEnum::Generic(ref data) => tpls.get("qr.html").unwrap().render(data),
                // And you may ask yourself: "Well, how did I get here?!"
                _ => {
                    panic!("Cosmic ray detected: Well, how did I get here?!")
                }
            };

            res.status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        e => {
            warn!(
                "[{:?} - {:?}] Bad Request Received: {}",
                headers[HOST].to_str().unwrap(),
                guess_ip(headers, remote_addr),
                e
            );
            // TODO: Nice Render page
            res.status_code(StatusCode::BAD_REQUEST);
        }
    }
}
